// addmzhdr, add mz header to a 16 bit .bin file to create an mz .exe file
// Copyright (C) 2012 neil foster <neil.foster@vampyre-adsl.demon.co.uk>
// 
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 1, or (at your option)
// any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#ifndef __inline_fns_h
#define __inline_fns_h

#include <stdarg.h>
#include "stdtypes.h"
#include "stddefs.h"
#ifdef WIN32
#include <sys/stat.h> // needed for s_iwusr
#endif

#ifdef _d_use_inline
static  inline void prints(char *s) {
  register char *ptr=s; 
    for(;*ptr;ptr++) 
    putc(*ptr); 
  }

static inline void term(int rc,char *str,...)  {
  char buf[255];
  va_list vl ;
  va_start(vl,str);
  vsnprintf(buf,sizeof(buf),str,vl);
  char *epos=strchr(buf,'\0');
   *epos='\n';
    epos[1]='\0';
  write(2,buf,strlen(buf));
  va_end(vl);
exit(rc);
}
#endif

#ifdef _d_use_inline
static  inline char *xstrdup(char *s) {
  register char *rc;
  if(!(rc=(void*)strdup(s)))
    term(1,"Cant strdup %s",s);
  return rc;
}
#endif

#ifdef _d_use_inline
static inline uc wordicmp(char *s1,char *s2) {
  for(;WORDCHAR(*s1)&&WORDCHAR(*s2);s1++,s2++)
  if(tolower(*s1)!=tolower(*s2)) break;

  if(WORDCHAR(*s1)) return 1;
  if(WORDCHAR(*s2)) return 1;

  return 0;
}

static inline 
int xopen(char *fn,int fl) {
  int mask=~O_BINARY;

   if(is_stdfile(fn)) {
     if(!(fl&mask)) return 1;
   return 0;
  }

#ifdef WIN32
  int fd=open(fn,fl,_S_IWUSR);
#else
  int fd=open(fn,fl);
#endif
  if(fd<0)
   term(1,"cant open %s flags=%x",fn,fl);
  return fd;
}
#endif

static inline void fprintnl(FILE *fh,char *b) {
   fprints(fh,b);
  fputc('\n',fh);
}
static inline void xfputc(int c,FILE *f) {
   if(fputc(c,f)==EOF)
    term(1,"Error in xfputc");
}

static  inline void *xmalloc(ul n) {
  void *rc;
  if(!(rc=(void*)malloc(n)))
    term(1,"Cant malloc %lu bytes",n);
  return rc;
}

#ifdef WIN32
static inline void xsysterm(int rc,char *msg,...) {
  char buf[80];
    DWORD err=GetLastError();
  va_list vl;
  va_start(vl,msg);

  vsnprintf(buf,sizeof(buf),msg,vl);
     systerm(rc,err,buf);
  va_end(vl);
}

static inline WNDCLASS *xreg_class(char *cl,HINSTANCE h,void *p,ul x) {
static WNDCLASS wc;
    wc.hCursor        = NULL;    // this window never shown, so no
    wc.hIcon          = NULL;    // cursor or icon are necessary
    wc.lpszMenuName   = NULL;
    wc.lpszClassName=cl;
    wc.hbrBackground  = (HBRUSH)(COLOR_WINDOW + 1);
    wc.hInstance=h;
    wc.style          = 0;
    wc.lpfnWndProc=p;
    wc.cbWndExtra=x;
    wc.cbClsExtra     = 0;
    if (!RegisterClass(&wc))
        term(1,"cant register calss");
  return &wc;
}

static inline HWND xparent_win(char *cl,char *wn,ul attr,HINSTANCE h) {
 HWND hwnd=CreateWindow(cl,wn,
  attr,
        0, 0, 0, 0,
        (HWND) NULL,        /* no parent */
        (HMENU) NULL,       /* use class menu */
        (HANDLE)h,    /* handle to window instance */
        (LPSTR) NULL        /* no params to pass on */
        );

    if(!hwnd)
   term(1,"cant create window");
   return hwnd;
}
static inline void xmkdir(char *a) {
 if(mkdir(a)) 
    term(1,"Error cant create dir %s",a);
}

static inline void xxReadFile(HANDLE fd,void *b,int l) {
  ul rc=xReadFile(fd,b,l);
  if(rc!=l)
  term(1,"xReadFile failed rc=%d should be %d",rc,l);
}

static inline void xxWriteFile(HANDLE fd,void *b,int l) {
  ul rc=xWriteFile(fd,b,l);
  if(rc!=l)
  term(1,"xWriteFile failed rc=%d should be %d",rc,l);
}

static inline int sp_get_category_from_id(void *id,void **cat,uc b) {
  int rc;
ISpObjectTokenCategory *c;

  if(rc=mkcomobj(SpObjectTokenCategory,&c))
  term(1,"co create failed");
  if(rc=ISpObjectTokenCategory_SetId(c,id,b))
  term(1," set id failed");
  *cat=c;
}

static inline int sp_enum_tokens(void *s,void *p1,void *p2,void **p3) {
  int rc;
ISpObjectTokenCategory *c;
  sp_get_category_from_id(s,(void**)&c,FALSE);
  return ISpObjectTokenCategory_EnumTokens(c,p1,p2,(void*)p3);
}
static inline void xEscapeCommFunction(HANDLE h,int fn) {
  if(!EscapeCommFunction(h,fn))
  xsysterm(1,"esc set");
}
static inline SC_HANDLE xCreateService(SC_HANDLE mgr,char *nm,char *disp,
int acc,int lvl,char *lcn,void *grp,void *dep) {
  SC_HANDLE h=OpenService(mgr,nm,acc);
  if(h) return h;
  h=CreateService(mgr,nm,disp,acc,
SERVICE_KERNEL_DRIVER,  // type of service
lvl,
                               SERVICE_ERROR_NORMAL,
lcn,
grp,
NULL,                   //no tag requested
dep,
NULL,                   // use LocalSystem account
NULL                    // no password for service account
                               );
  if(!h)
   xsysterm(1,"create svc failed");
   return h;
}
static inline HANDLE xCreateThread(void *fn,void *p,int *id) {
   HANDLE thread;
 if(!(thread=CreateThread(NULL,0,(void*)fn,
p,STANDARD_RIGHTS_REQUIRED,(void*)id)))
  xsysterm(1,"thread create failed");
     return thread;
}
static inline xSetThreadPriority(HANDLE h,int p) {
  if(!SetThreadPriority(h,p))
   xsysterm(1,"failed setting thread priority");
}
#endif

#ifdef _d_use_sys_hack
static inline __attribute__((regparm(2))) sys_hack(ul ebx,ul ecx,...) {
   ul edx=0;

  va_list vl;
  va_start(vl,ecx);
  if(ebx==hack_read_ctl_reg)
    edx=va_arg(vl,ul);
  va_end(vl);

  asm("mov %0,%%ebx"::"m"(ebx));
  asm("mov %0,%%ecx"::"m"(ecx));
  asm("mov %0,%%edx"::"m"(edx));
  asm("mov %0,%%eax"::"n"(__NR_hack));
 asm("int $0x80");
}
#endif

static  inline void xxread(int fd,void *b,int l) {
  ul rc=read(fd,b,l);
  if(rc!=l)
  term(1,"read failed rc=%d should be %d",rc,l);
}

static inline void xxwrite(int fd,void *b,int l) {
  ul rc=write(fd,b,l);
  if(rc!=l)
  term(1,"write failed rc=%d should be %d",rc,l);
}
#endif
