// addmzhdr, add mz header to a 16 bit .bin file to create an mz .exe file
// Copyright (C) 2012 neil foster <neil.foster@vampyre-adsl.demon.co.uk>
// 
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 1, or (at your option)
// any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#ifndef _STDTYPES_H
#define _STDTYPES_H

#ifdef __GNUC__
#define rcall(a) __attribute__((regparm(a)))
#define regcall __attribute__((regparm(0))
#define stdcall __attribute__((__stdcall__))
#define attr_packed __attribute__((packed))
#define align(n) __attribute((aligned(n)))
#define section(a) __attribute__((section(#a)))
#else
#define attr_packed
#define stdcall
#endif

#define iregcall inline __attribute__((regparm(0)))
#ifndef _d_max_path
#define _d_max_path 260
#endif

typedef unsigned char uc;
typedef unsigned short us;
typedef unsigned int ui;
typedef unsigned long ul;

#ifdef __DJGPP__
  typedef char BYTE;
  typedef short WORD;
  typedef long DWORD;
typedef us *PWCHAR;
#endif

typedef struct {
  us seg_limit;
  uc base_addr[3];
  uc type;
  uc  f;
  uc base_addr_1;
} desc_t;

#ifdef __GNUC__
typedef long long quad;
typedef unsigned long long uq;
#endif
typedef int (*fooptr)();

#define rguc register uc
#define rgchar register char
#define rgint register int
#define rgui register ui
#define rgus register us
#define rgul register ul

#ifndef _TURBOC_
typedef  struct {
 ul fsz;
  ul res;
  ul offbits;
  ui biSize;                 //size of struct
  int biWidth;                 //image width
  int biHeight;                //image height
  us biPlanes;               //1
  us biBitCount;             //bpp
  ui biCompression;          //0 for uncompressed
  ui biSizeImage;            //image size
  int biXPelsPerMeter;         //res in pp metre
  int biYPelsPerMeter;
  ui biClrUsed;              //0 or actual size of colour table
  ui biClrImportant;         //usually 0
} bmp_t;

typedef struct image_dos_hdr_s {
	us e_magic;
	us e_cblp;
	us e_cp;
	us e_crlc;
	us e_cparhdr;
	us e_minalloc;
	us e_maxalloc;
	us e_ss;
	us e_sp;
	us e_csum;
	us e_ip;
	us e_cs;
	us e_lfarlc;
	us e_ovno;
	us e_res[4];
	us e_oemid;
	us e_oeminfo;
	us e_res2[10];
	long e_lfanew;
} attr_packed image_dos_hdr_t;

typedef struct dex_hdr_s {
 char n[7];
 ul h,s,c,start_img;
} attr_packed dex_hdr_t;

typedef struct {
	uc	name[8],ext[3];	/* name and extension */
	uc	attr;		/* attribute bits */
	uc    lcase;		/* Case for base and extension */
	uc	ctime_ms;	/* Creation time, milliseconds */
	us	ctime;		/* Creation time */
	us	cdate;		/* Creation date */
	us	adate;		/* Last access date */
	us   starthi;	/* High 16 bits of cluster in FAT32 */
	us	time,date,start;/* time, date and first cluster */
	ul	size;		/* file size (in bytes) */
} attr_packed fat_dir_entries_t;

typedef struct {
                uc id;               // sequence number for slot 
                uc name0_4[10];      // first 5 characters in name 
                us attr;             // attribute byte
  uc alias_chksum;   // checksum for 8.3 alias 
                uc name5_10[12];     // 6 more characters in name
                uc start[2];         // starting cluster number
                uc name11_12[4];     // last 2 characters in name
} attr_packed fat_slot_t;

typedef struct {
#if __GNUC__ >= 3
  union {
  struct { 
#endif
	uc	ignored[3];	/* Boot strap short or near jump */
	uc	system_id[8];	/* Name - can be used to special case
				   partition manager volumes */
	us	sector_size;	/* bytes per logical sector */
	uc	sec_per_clus;	/* sectors/cluster */
	us	reserved;	/* reserved sectors */
	uc	fats;		/* number of FATs */
	us	dir_entries;	/* root directory entries */
	us	sectors;	/* number of sectors */
	uc	media;		/* media code */
	us	fat_length;	/* sectors/FAT */
	us	secs_track;	/* sectors per track */
	us	heads;		/* number of heads */
	ul	hidden;		/* hidden sectors (unused) */
	ul	total_sect;	/* number of sectors (if sectors == 0) */

	/* The following fields are only used by FAT32 */
	ul	fat32_length;	/* sectors/FAT */
	us	flags;		/* bit 8: fat mirroring, low 4: active fat */
	us	version;	/* major, minor filesystem version */
	ul	root_cluster;	/* first cluster in root directory */
	us	info_sector;	/* filesystem info sector */
	us	backup_boot;	/* backup boot sector */
	us	reserved2[6];	/* Unused */
  uc boot_drive;
#if __GNUC__ < 3
  char padding[447];
#else
  } attr_packed;
  char b[512];
};
#endif
} attr_packed  fat_boot_sec_t;
#endif

typedef struct _finddata_t finddata_t;
#if defined(WIN32)&&defined(_IO_H_)
typedef void *port_t;
typedef void *os_fd_t;
#ifdef _WINBASE_H
typedef PROCESS_INFORMATION process_information_t;
#endif
typedef struct {
void *h;
  ul p;
} _usbdev_t,*usbdev_t;
#else
typedef int port_t;
typedef int os_fd_t;
typedef struct {
  os_fd_t hProcess;
}  process_information_t;
struct _finddata_t
{
	unsigned	attrib;		/* Attributes, see constants above. */
	ul		time_create;
	ul		time_access;	/* always midnight local time */
	ul		time_write;
	uq	size;
	char		name[_d_max_path];	/* may include spaces. */
};
#endif

typedef struct {
  char dir[_d_max_path];
  ul n;
  finddata_t fd;
  ul mode;
   ul level;
} treewalk_t;

typedef struct {
   ul media;
   ul cyls;
   us heads,secs;
  us sec_sz;
}  drive_parms_t;
typedef struct {
 uc h,s,c;
} attr_packed hsc_t;

typedef struct {
  uc active;
  hsc_t start_hsc;
   uc os;
  hsc_t end_hsc;
  ul offset;
   ul sectors;
} attr_packed partition_record_t;

typedef struct {
  char c[0x1be];
  partition_record_t pr[4];
  us sig;
} attr_packed mbr_t;

typedef struct nmalloc_s { 
struct nmalloc_s *next;
ul size; 
} nmalloc_t;

#ifndef _TURBOC_
typedef struct sockaddr_in sockaddr_in_t;
#ifdef _d_ancient_regieme
typedef struct in_addr in_addr_t;
#endif
typedef struct timeval timeval_t;
typedef struct timezone timezone_t;
#endif
typedef int(*fn_ptr_t)();

typedef struct {
   char *ptr;
  char *nxt;
  char *rem;
} bufread_t;

typedef struct {
uc length;
uc type;
us rev;
  uc class[2];
uc protocol;
uc max_pkt_sz;
us vendor;
us product;
us release;
uc imft;
uc iproduct;
uc iserial;
uc num_cfgs;
} attr_packed std_dev_descriptor_t;

typedef struct {
 uc length;
  uc descriptor_type;
us total_length;
  uc total_num_interfaces;
  uc cfg_value;
 uc icfg;
  uc attributes;
  uc max_power;
} attr_packed std_cfg_descriptor_t;

typedef struct {
 uc len;
  uc type;
  uc intf_num;
  uc alt_setting;
  uc n_endpts;
  uc  class[2];
  uc protocol;
  uc iintf;
} attr_packed std_intf_descriptor_t;

typedef struct {
 uc length;
  uc descriptor_type;
  uc addr;
  uc attrib;
  us max_packet_sz;
  uc poll_interval;
} attr_packed std_endpoint_descriptor_t;

typedef struct stat stat_t;
#if defined(WIN32)||defined(__DJGPP__)
typedef struct stat stat64_t;
#else
struct _13_stat64 {
	unsigned short	st_dev;
	unsigned char	__pad0[10];
unsigned long	st_ino;
	unsigned int	st_mode;
	unsigned int	st_nlink;

	unsigned long	st_uid;
	unsigned long	st_gid;

	unsigned short	st_rdev;
	unsigned char	__pad3[10];

	long long	st_size;
	unsigned long	st_blksize;

	unsigned long	st_blocks;	/* Number 512-byte blocks allocated. */
	unsigned long	__pad4;		/* future possible st_blocks high bits */

	unsigned long	st_atime;
	unsigned long	__pad5;

	unsigned long	st_mtime;
	unsigned long	__pad6;

	unsigned long	st_ctime;
	unsigned long	__pad7;		/* will be high 32 bits of ctime someday */

	unsigned long	__unused1;
	unsigned long	__unused2;
};
typedef struct _13_stat64 stat64_t;
#endif

typedef struct statfs statfs_t;
typedef struct dirent dirent_t;

typedef struct{
  char  riff[4];            /* must be "RIFF"                */
  ul dwLen;                /* #bytes + 44 - 8               */
  char  cWavFmt[8];         /* must be "WAVEfmt"             */
  ul dwHdrLen; // 0x10
  us  wFormat; // 0x14
  us  wNumChannels;  // 0x16
  ul dwSampleRate; // 0x18
  ul dwBytesPerSec; // 0x1c
  us  wBlockAlign; //0x20
  us  wBitsPerSample;
  char  cData[4];            /* must be "data"               */
  ul dwDataLen;           /* #bytes                       */
} wavfilehdr_t;

typedef  struct{ 
	us	i_mode;		/* File mode */
	us	i_uid;		/* Low 16 bits of Owner Uid */
	ul	i_size;		/* Size in bytes */
	ul	i_atime;	/* Access time */
	ul	i_ctime;	/* Creation time */
	ul	i_mtime;	/* Modification time */
	ul	i_dtime;	/* Deletion Time */
	us	i_gid;		/* Low 16 bits of Group Id */
	us	i_links_count;	/* Links count */
	ul	i_blocks;	/* Blocks count */
	ul	i_flags;	/* File flags */
	union {
		struct {
			ul  l_i_reserved1;
		} linux1;
		struct {
			ul  h_i_translator;
		} hurd1;
		struct {
			ul  m_i_reserved1;
		} masix1;
	} osd1;				/* OS dependent 1 */
#ifndef _d_ext2_n_blocks
#define _d_ext2_n_blocks 15
#endif
	ul	i_block[_d_ext2_n_blocks];/* Pointers to blocks */
	ul	i_generation;	/* File version (for NFS) */
	ul	i_file_acl;	/* File ACL */
	ul	i_dir_acl;	/* Directory ACL */
	ul	i_faddr;	/* Fragment address */
	union {
		struct {
			uc	l_i_frag;	/* Fragment number */
			uc	l_i_fsize;	/* Fragment size */
			us	i_pad1;
			us	l_i_uid_high;	/* these 2 fields    */
			us	l_i_gid_high;	/* were reserved2[0] */
			ul	l_i_reserved2;
		} linux2;
		struct {
			uc	h_i_frag;	/* Fragment number */
			uc	h_i_fsize;	/* Fragment size */
			us	h_i_mode_high;
			us	h_i_uid_high;
			us	h_i_gid_high;
			ul	h_i_author;
		} hurd2;
		struct {
			uc	m_i_frag;	/* Fragment number */
			uc	m_i_fsize;	/* Fragment size */
			us	m_pad1;
			ul	m_i_reserved2[2];
		} masix2;
	} osd2;				/* OS dependent 2 */
} _13_ext2_inode_t;

typedef struct {
	ul	s_inodes_count;		/* Inodes count */
	ul	s_blocks_count;		/* Blocks count */
	ul	s_r_blocks_count;	/* Reserved blocks count */
	ul	s_free_blocks_count;	/* Free blocks count */
	ul	s_free_inodes_count;	/* Free inodes count */
	ul	s_first_data_block;	/* First Data Block */
	ul	s_log_block_size;	/* Block size */
	long	s_log_frag_size;	/* Fragment size */
	ul	s_blocks_per_group;	/* # Blocks per group */
	ul	s_frags_per_group;	/* # Fragments per group */
	ul	s_inodes_per_group;	/* # Inodes per group */
	ul	s_mtime;		/* Mount time */
	ul	s_wtime;		/* Write time */
	us	s_mnt_count;		/* Mount count */
	short	s_max_mnt_count;	/* Maximal mount count */
	us	s_magic;		/* Magic signature */
	us	s_state;		/* File system state */
	us	s_errors;		/* Behaviour when detecting errors */
	us	s_minor_rev_level;	/* minor revision level */
	ul	s_lastcheck;		/* time of last check */
	ul	s_checkinterval;	/* max. time between checks */
	ul	s_creator_os;		/* OS */
	ul	s_rev_level;		/* Revision level */
	us	s_def_resuid;		/* Default uid for reserved blocks */
	us	s_def_resgid;		/* Default gid for reserved blocks */
	/*
	 * These fields are for EXT2_DYNAMIC_REV superblocks only.
	 *
	 * Note: the difference between the compatible feature set and
	 * the incompatible feature set is that if there is a bit set
	 * in the incompatible feature set that the kernel doesn't
	 * know about, it should refuse to mount the filesystem.
	 *
	 * e2fsck's requirements are more strict; if it doesn't know
	 * about a feature in either the compatible or incompatible
	 * feature set, it must abort and not try to meddle with
	 * things it doesn't understand...
	 */
	ul	s_first_ino;		/* First non-reserved inode */
	us   s_inode_size;		/* size of inode structure */
	us	s_block_group_nr;	/* block group # of this superblock */
	ul	s_feature_compat;	/* compatible feature set */
	ul	s_feature_incompat;	/* incompatible feature set */
	ul	s_feature_ro_compat;	/* readonly-compatible feature set */
	uc	s_uuid[16];		/* 128-bit uuid for volume */
	char	s_volume_name[16];	/* volume name */
	char	s_last_mounted[64];	/* directory where last mounted */
	ul	s_algorithm_usage_bitmap; /* For compression */
	/*
	 * Performance hints.  Directory preallocation should only
	 * happen if the EXT2_FEATURE_COMPAT_DIR_PREALLOC flag is on.
	 */
	uc	s_prealloc_blocks;	/* Nr of blocks to try to preallocate*/
	uc	s_prealloc_dir_blocks;	/* Nr to preallocate for dirs */
	us	s_reserved_gdt_blocks;	/* Per group table for online growth */
	/*
	 * Journaling support valid if EXT2_FEATURE_COMPAT_HAS_JOURNAL set.
	 */
	uc	s_journal_uuid[16];	/* uuid of journal superblock */
	ul	s_journal_inum;		/* inode number of journal file */
	ul	s_journal_dev;		/* device number of journal file */
	ul	s_last_orphan;		/* start of list of inodes to delete */
	ul	s_hash_seed[4];		/* HTREE hash seed */
	uc	s_def_hash_version;	/* Default hash version to use */
	uc	s_jnl_backup_type; 	/* Default type of journal backup */
	us	s_reserved_word_pad;
	ul	s_default_mount_opts;
	ul	s_first_meta_bg;	/* First metablock group */
	ul	s_mkfs_time;		/* When the filesystem was created */
	ul	s_jnl_blocks[17]; 	/* Backup of the journal inode */
	ul	s_reserved[172];	/* Padding to the end of the block */
} _13_ext2_super_block_t;

typedef struct {
	ul	bg_block_bitmap;		/* Blocks bitmap block */
	ul	bg_inode_bitmap;		/* Inodes bitmap block */
	ul	bg_inode_table;		/* Inodes table block */
	us	bg_free_blocks_count;	/* Free blocks count */
	us	bg_free_inodes_count;	/* Free inodes count */
	us	bg_used_dirs_count;	/* Directories count */
	us	bg_pad;
	ul	bg_reserved[3];
} _13_ext2_group_desc_t;

typedef struct pnp_install_s {
ul sig;
char Plug_and_Play_version;
char len_struct;
us control_field;
char checksum;
ul event_phys_addr;
us rmode_offs;
us rmode_cseg;
us pmode16_cseg_offs;
ul pmode16_cseg_base_addr;
ul OEM_device_identifier;
us rmode_dseg;
ul pmode16_dseg_base_addr;
} attr_packed pnp_install_t;
typedef struct {
  char mthd[4]; // MThd
  ul f6,divs;
 us fe1; // has value e01
} attr_packed midi0_hdr_t;

typedef struct {
  char mtrk[4]; // MTrk
  ul len;
   char data[1];
} attr_packed midi0_trk_t;

typedef struct {
  us l;
 ul b;
} attr_packed mem_man_reg_t;

typedef struct image_file_header_s{
	short Machine;
	short NumberOfSections;
	long TimeDateStamp;
	long PointerToSymbolTable;
	long NumberOfSymbols;
	short SizeOfOptionalHeader;
	short Characteristics;
} attr_packed image_file_header_t;

typedef struct image_data_directory_t{
	long VirtualAddress;
	long Size;
} attr_packed image_data_directory_t;

typedef struct image_optional_header_s{
	short Magic;
	char MajorLinkerVersion;
	char MinorLinkerVersion;
	long SizeOfCode;
	long SizeOfInitializedData;
	long SizeOfUninitializedData;
	long AddressOfEntryPoint;
	long BaseOfCode;
	long BaseOfData;
	long ImageBase;
	long SectionAlignment;
	long FileAlignment;
	short MajorOperatingSystemVersion;
	short MinorOperatingSystemVersion;
	short MajorImageVersion;
	short MinorImageVersion;
	short MajorSubsystemVersion;
	short MinorSubsystemVersion;
	long Win32VersionValue;
	long SizeOfImage;
	long SizeOfHeaders;
	long CheckSum;
	short Subsystem;
	short DllCharacteristics;
	long SizeOfStackReserve;
	long SizeOfStackCommit;
	long SizeOfHeapReserve;
	long SizeOfHeapCommit;
	long LoaderFlags;
	long NumberOfRvaAndSizes;
	image_data_directory_t DataDirectory[IMAGE_NUMBEROF_DIRECTORY_ENTRIES];
} attr_packed image_optional_header_t;

typedef struct image_section_header_s{
	char Name[IMAGE_SIZEOF_SHORT_NAME];
	union {
		long PhysicalAddress;
		long VirtualSize;
	} Misc;
	long VirtualAddress;
	long SizeOfRawData;
	long PointerToRawData;
	long PointerToRelocations;
	long PointerToLinenumbers;
	short NumberOfRelocations;
	short NumberOfLinenumbers;
	long Characteristics;
} attr_packed image_section_header_t;
typedef struct image_export_directory_s {
	long Characteristics;
	long TimeDateStamp;
	short MajorVersion;
	short MinorVersion;
	long Name;
	long Base;
	long NumberOfFunctions;
	long NumberOfNames;
	long AddressOfFunctions;
	long AddressOfNames;
	long AddressOfNameOrdinals;
} attr_packed image_export_directory_t;

#endif

